# -*- coding: utf-8 -*-
"""
Simula el movimiento de una esfera en un plano inclinado

Se entiende que la esfera rueda sin deslizarse
Utilizamos unidades del sistema internacional (kg, m, s, J)
Para el ángulo utilizamos grados
Ecuaciones del movimiento en : http://www.sc.ehu.es/sbweb/fisica/

@author: Michael
@date:   Ene 2019
"""
import math
from typing import Final
import matplotlib.pyplot as plt
import numpy

# Constantes:
# G : gravedad en m*s**2

G: Final[float] = 9.8  # metros/seg2


class PlanoInclinado:
    """
    Sistema Esfera - Plano Inclinado
    Contiene los datos de un sistema formado por
    una esfera y un plano inclinado
    y métodos para hacer cálculos de trayectorias
    y energías

    Atributes:
        __masa:   masa de la esfera, en kg
        __radio:  radio de la esfera, en m
        __angulo: ángulo del plano inclinado, en grados
    """

    def __init__(self, masa: float, radio: float,
                 angulo: float):
        """
        Constructor del sistema esfera - plano inclinado

        Args:
            masa:   masa de la esfera, en kg
            radio:  radio de la esfera, en m
            angulo: ángulo del plano inclinado, en grados

        """

        self.__masa = masa
        self.__radio = radio
        self.__angulo = angulo

    def momento_inercia(self) -> float:
        """
        Calcula el momento de inercia de la esfera

        Returns:
            el momento de indercia de la esfera, en kg*m**2
        """

        return 2.0*self.__masa*self.__radio**2/5.0

    def aceleracion(self) -> float:
        """
        Calcula la aceleración lineal del objeto
        Returns:
            la aceleración de la esfera en m/s
        """

        return (G*math.sin(math.radians(self.__angulo)) /
                (1+self.momento_inercia() /
                 (self.__masa*self.__radio**2)))

    def distancia(self, tiempo: float) -> float:
        """
         Calcula la distancia recorrida por el
         objeto en el tiempo indicado

        Args:
            tiempo transcurrido, en s
        Returns:
            la distancia recorrida por la esfera
        """

        return self.aceleracion()*tiempo**2/2.0

    def e_cinetica_tras(self, tiempo: float) -> float:
        """
        Calcula la energía cinética de traslación,
        del objeto transcurrido el tiempo indicado (Jul)

        Args:
            tiempo transcurrido, en s
        Returns:
            la energía cinética de traslación de la esfera en J
        """

        vel = self.aceleracion()*tiempo
        return self.__masa*vel**2/2.0

    def e_cinetica_rot(self, tiempo: float) -> float:
        """
        Calcula la energía cinética de rotación,
        del objeto transcurrido el tiempo indicado (Jul)

        Args:
            tiempo transcurrido, en s
        Returns:
            la energía cinética de rotación de la esfera en J
        """

        vel_angular: float = \
            self.aceleracion()*tiempo/self.__radio
        return self.momento_inercia()*vel_angular**2/2.0


def main():
    """
    Programa principal que muestra para una esfera en
    un plano inclinado la distancia y energías a los
    cuatro segundos
    """

    # Crear el sistema plano-esfera
    # Ángulo 30 grados, esfera de 1.5 kg y r=0.2 m
    plano = PlanoInclinado(masa=1.5, radio=0.2, angulo=30)

    # Mostrar resultados

    print("Dist. a los 4 seg: " +
          f"{plano.distancia(4.0)} m")
    print("E. C. tras. a 4 seg: " +
          f"{plano.e_cinetica_tras(4.0):.3f} J")
    print("E. C. rot a 4 seg: " +
          f"{plano.e_cinetica_rot(4.0):.3f} J")


def simula():
    """
    Programa principal que simula para una esfera en
    un plano inclinado las energías en función de las
    distancias recorridas
    """

    # Crear el sistema plano-esfera
    # Ángulo 30 grados, esfera de 1.5 kg y r=0.2 m
    plano = PlanoInclinado(masa=1.5, radio=0.2, angulo=30)

    # Creamos unas listas para distancias y energías
    list_dist = [0.0]
    list_e_tras = [0.0]
    list_e_rot = [0.0]

    # Mostrar simulación de las energías entre 0 y 4 seg
    for tiempo in numpy.linspace(0.0, 4.0, num=100):
        list_dist.append(plano.distancia(tiempo))
        list_e_tras.append(plano.e_cinetica_tras(tiempo))
        list_e_rot.append(plano.e_cinetica_rot(tiempo))

    # Creamos la grafica para dibujar las trayectorias
    plt.plot(list_dist, list_e_tras, "red", label="Enegía traslación")
    plt.plot(list_dist, list_e_rot, "blue", label="Energía rotación")
    plt.ylabel("Energía (J)")
    plt.xlabel("Dist(m)")
    plt.title("Energía cinética de una esfera en un plano inclinado")
    plt.grid(True)
    plt.legend()
    plt.show()


def ejemplos():
    """
    Ejemplos de creación de objetos
    """

    plano_1 = PlanoInclinado(1.5, 0.2, 30)
    plano_2 = PlanoInclinado(masa=1.5, radio=0.2, angulo=40)
    print("Dist. 1 a los 10 seg: "+str(plano_1.distancia(10.0)))
    print("Dist. 2 a los 10 seg: "+str(plano_2.distancia(10.0)))
